<div unicorn:poll-1000="get_date" unicorn:poll.disable="polling_disabled">
    {% if messages %}
    <ul class="messages">
        {% for message in messages %}
        <li{% if message.tags %} class="{{ message.tags }}"{% endif %}>{{ message }}</li>
        {% endfor %}
    </ul>
    {% endif %}

    <div id="poll-id">
        poll-id: {{ current_time|date:"s" }}
    </div>

    current_time: {{ current_time|date:"s" }}

    <br /><br/>
    <button u:click="slow_update()">Increase counter: {{counter}}</button>

    <br /><br />
    <input unicorn:model="date_example" type="text" id="dateExampleId">
    date_example: {{ date_example }} | {{ date_example|date:"s" }}<br />

    <button u:click="polling_disabled=False">Enable Polling</button>
    <button u:click="polling_disabled=True">Disable Polling</button>
    <button u:click="$toggle('polling_disabled')">Toggle Polling</button>
</div>
