{% extends "www/base.html" %}
{% load static unicorn %}

{% block content %}

<h2>Text inputs</h2>

<button onclick="Unicorn.call('zxcv', 'set_name');">Method call from outside of the component</button>

{% unicorn 'text-inputs' key="asdf" %}

{% unicorn 'text-inputs' key="zxcv" %}

{% endblock content %}