# Generated by Django 3.1.5 on 2021-01-28 01:40

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('coffee', '0002_auto_20201205_1450'),
    ]

    operations = [
        migrations.AddField(
            model_name='flavor',
            name='date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='flavor',
            name='datetime',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='flavor',
            name='duration',
            field=models.DurationField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='flavor',
            name='time',
            field=models.TimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='flavor',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4),
        ),
        migrations.AlterField(
            model_name='flavor',
            name='decimal_value',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=10, null=True),
        ),
        migrations.AlterField(
            model_name='flavor',
            name='float_value',
            field=models.FloatField(blank=True, null=True),
        ),
    ]
